#include "mbed.h"
#include "w5uxhMbedKeyer.h"

void setupKbdDash(void);
void setupKbdDot(void);
void setupKbdElementSpace(void);
void setupKbdCharSpace(void);

Timeout cwTimer;
DigitalOut rig1(p11);


#define KEY_UP   0
#define KEY_DOWN 1

extern PwmOut sidetone;

int keyState = KEY_UP;

#define KBD    0
#define PDL    1
#define WEIGHT 2

#define KBD_IDLE       0
#define KBD_DOT        1
#define KBD_DASH       2
#define KBD_EL_SPACE   3
#define KBD_CHAR_SPACE 4
#define KBD_WORD_SPACE 5

unsigned char cwGenerateMode = KBD;
unsigned char arrowsControl  = KBD;
unsigned char kbdCwState;

volatile unsigned char workingMorseByte;

extern unsigned int kbdDotLength_usec;
extern unsigned int kbdDashLength_usec;
extern unsigned int kbdElSpace_usec;
extern unsigned int kbdCharSpace_usec;
extern unsigned int kbdWordSpace_usec;

extern Serial pc;

/** -------------------------------------------------------------------------
@verbatim
This is the common timer interrupt routine.  It checks to see if KBD or PDL is the active source
of the cw and calls the appropriate handler.
cwTimer is attached at the start of every element or space, with the specified timout in usecs.

@endverbatim
**/

void atCwTimer(void)
{
  if (cwGenerateMode == KBD)
    {
      atCwTimerKbd();
    }
  else
    {
      atCwTimerPdl();
    }
}

/** -------------------------------------------------------------------------
@verbatim
This is the timer interrupt routine for the keyboard. It is called from atCwTimer() if the kbd
is the active source of the cw. 
cwTimer is attached at the start of every element or space, with the specified timout in usecs.

@endverbatim
**/

void atCwTimerKbd(void)
{

  if ((kbdCwState != KBD_WORD_SPACE) && (kbdCwState != KBD_CHAR_SPACE))
    {
      if (workingMorseByte != 0x80) // <-- 0x80 flags end of character
    {
      // Not end of character
      switch(kbdCwState)
        {
        case KBD_DOT:  case KBD_DASH:
          {
        // Following every DOT or DASH element, we shift the working byte
        // to prepare for the next element.
        workingMorseByte <<= 1;
        setupKbdElementSpace();
        break;
          }

        default:
          // If the interval just finished was not a DOT or DASH, then we
          // are ready to check the next element to setup if this is not
          // the end of the character.
          {
        if (workingMorseByte & 0x80)
          {
            setupKbdDash();
          }
        else
          {
            setupKbdDot();
          }
        break;
          }

        }  // end:  switch(kbdCwState)


    }
      else // alternative to: if (workingMorseByte != 0x80)
    {
      // End of character
      setupKbdCharSpace();
    } // end:  if (workingMorseByte != 0x80)
    

    } // end:  if ((kbdCwState != KBD_WORD_SPACE) && (kbdCwState != KBD_CHAR_SPACE))
  else
    {
      kbdCwState = KBD_IDLE;
    }
}

bool rigEna = TRUE;
bool monEna = TRUE;

/** -------------------------------------------------------------------------
@verbatim
Key down

@endverbatim
**/
void keyDown()
{
  if (rigEna)
    {
      rig1 = KEY_DOWN;
    }

  if (monEna)
    {
      sidetone = 0.5;
    }

  keyState = KEY_DOWN;
}

/** -------------------------------------------------------------------------
@verbatim
Key up

@endverbatim
**/
void keyUp()
{
  if (rigEna)
    {
      rig1 = KEY_UP;
    }

  if (monEna)
    {
      sidetone = 0.0;
    }

  keyState = KEY_UP;
}

/** -------------------------------------------------------------------------
@verbatim
Setup to send a dash (from keyboard)

@endverbatim
**/
void setupKbdDash(void)
{
  keyDown();
  cwTimer.attach_us(&atCwTimer, kbdDashLength_usec);
  kbdCwState = KBD_DASH;
}

/** -------------------------------------------------------------------------
@verbatim
Setup to send a dot (from keyboard)

@endverbatim
**/
void setupKbdDot(void)
{
  keyDown();
  cwTimer.attach_us(&atCwTimer, kbdDotLength_usec);
  kbdCwState = KBD_DOT;
}

/** -------------------------------------------------------------------------
@verbatim
Setup element space

@endverbatim
**/
void setupKbdElementSpace(void)
{
  keyUp();
  cwTimer.attach_us(&atCwTimer, kbdElSpace_usec);
  kbdCwState = KBD_EL_SPACE;
}


/** -------------------------------------------------------------------------
@verbatim
Setup character space

@endverbatim
**/
void setupKbdCharSpace(void)
{
  keyUp();
  cwTimer.attach_us(&atCwTimer, kbdCharSpace_usec);
  kbdCwState = KBD_CHAR_SPACE;
}

/** -------------------------------------------------------------------------
@verbatim
Setup word space

@endverbatim
**/
void setupKbdWordSpace(void)
{
  keyUp();
  cwTimer.attach_us(&atCwTimer, kbdWordSpace_usec);
  kbdCwState = KBD_WORD_SPACE;
}




/** -------------------------------------------------------------------------
@verbatim
Prepare to start generating the CW character.

Coding example for the BT character:

    0b10001100,    //   BT

 1   0   0   0   1    1
dah dit dit dit dah  end

@endverbatim
**/
void startCwCharacter(unsigned char morseByte)
{
  if (cwGenerateMode != KBD)
    {
      cwGenerateMode = KBD;
    }


  //0xB6 is the control key code for the End key
  checkAutoFocus(KBD);


  workingMorseByte = morseByte;

  // Word space is coded as 0x00
  if (morseByte != 0x00)
    {
      if (workingMorseByte & 0x80)
    {
      setupKbdDash();
    }
      else
    {
      setupKbdDot();
    }
    }
  else
    {
      setupKbdWordSpace();
    }

}
