#include "mbed.h"
#include "TextLCD.h"
#include "w5uxhMbedKeyer.h"
#include "queues.h"

extern int cwBufSize;
extern Fifo *cwBuf;

#define ROW0 0
#define ROW1 1
#define ROW2 2
#define ROW3 3

#define COL0 0
#define COL8 8
#define COL9 9
#define COL16 16

// The mbed locate function for the LCD is COL,ROW starting from col or row 0:
#define TIMEcursor COL0,ROW1
#define PDLcursor  COL0,ROW0
#define KBDcursor  COL8,ROW0
#define WTcursor   COL9,ROW1
#define BUFfreeCursor COL16, ROW0
#define BUFusedCursor COL16, ROW1

extern TextLCD lcd;
extern unsigned char kbdWPM;
extern unsigned char pdlWPM;
extern unsigned char weight;

bool toggleSameSpeedModeState = FALSE;

int line2Column=0;
char line2Buffer[20];
int line2BufIndex=0;

int line3Column=0;
char line3Buffer[20];
int line3BufIndex=0;

unsigned char lcdDisplayState = ROW2_ONLY;

/** -------------------------------------------------------------------------

@verbatim
These three characters are used on the LCD display to show where the arrow key
focus is aimed.  All three chars need to be updated each time the focus changes.

@endverbatim
**/
unsigned char pdlFocusChar = ':';
unsigned char kbdFocusChar = '=';
unsigned char wtFocusChar  = ':';

unsigned char focus = KBD;

/** -------------------------------------------------------------------------

@verbatim
Display buffer status

@endverbatim
**/
void dsplyBufStatusLCD(void)
{
  int free;
  int used;

  used = cwBuf->get_count();
  free = cwBufSize - used;

  lcd.locate(BUFfreeCursor);
  lcd.printf("%4d", free);
  lcd.locate(BUFusedCursor);
  lcd.printf("%4d", used);
}

/** -------------------------------------------------------------------------

@verbatim
Display speed and weight settings.

@endverbatim
**/
void displaySpeedsLCD(void)
{
  lcd.locate(PDLcursor);
  lcd.printf("PDL%c%2d", pdlFocusChar, pdlWPM);

  lcd.locate(KBDcursor);
  lcd.printf("KBD%c%3d", kbdFocusChar, kbdWPM);

  lcd.locate(WTcursor);
  lcd.printf("WT%c%2d%%", wtFocusChar, weight);


}


/** -------------------------------------------------------------------------

@verbatim
Check speed limits for paddle

@endverbatim
**/
bool pdlWpmLimitOK(int delta)
{
  int tempValue;

  tempValue = pdlWPM+delta;

  if (((tempValue) > 5) && ((tempValue < 100)))
    {
      return TRUE;
    }
  else
    {
      return FALSE;
    }
}


/** -------------------------------------------------------------------------

@verbatim
Check speed limits for kbd

@endverbatim
**/
bool kbdWpmLimitOK(int delta)
{
  int tempValue;

  tempValue = kbdWPM+delta;

  if (((tempValue) > 5) && ((tempValue < 125)))
    {
      return TRUE;
    }
  else
    {
      return FALSE;
    }
}


/** -------------------------------------------------------------------------

@verbatim
Change kbd speed as a function of the arrow key control.

@endverbatim
**/
void deltaSpeedKbd(int delta)
{
  if (kbdWpmLimitOK(delta))
    {
 
      kbdWPM += delta;
      calcTimeDurationsKbd(kbdWPM, weight);
      displaySpeedsLCD();

      if (toggleSameSpeedModeState)
    {
      pdlWPM += delta;
      calcTimeDurationsPdl(pdlWPM, weight);
      displaySpeedsLCD();
    }
    }
}


/** -------------------------------------------------------------------------

@verbatim
Change pdl speed as a function of the arrow key control.

@endverbatim
**/

void deltaSpeedPdl(int delta)
{
  if (pdlWpmLimitOK(delta))
    {
      pdlWPM += delta;
      calcTimeDurationsPdl(pdlWPM, weight);
      displaySpeedsLCD();
      
      if (toggleSameSpeedModeState)
    {
      kbdWPM += delta;
      calcTimeDurationsKbd(kbdWPM, weight);
      displaySpeedsLCD();
    }
    }
}


/** -------------------------------------------------------------------------
@verbatim
Scroll string text to the left on the specified row.

@endverbatim
**/
void displayLcdStringRow(unsigned char row, char *string)
{
  int length;
  int i;

  length= strlen(string);
  
  for(i=0; i<length; i++)
    {
      displayLcdTextRow(row, string[i]);
    }

}


/** -------------------------------------------------------------------------
@verbatim
Scroll one char to the left on the specified row.

@endverbatim
**/
void displayLcdTextRow(unsigned char row, unsigned char newChar)
{
  unsigned char srcIndex;
  unsigned char destIndex;
  unsigned char backspace;
  int column;
  char *lineBuffer;
  int colIndex;
  char tempString[2];

  if (lcdDisplayState == NO_TEXT) return;

  switch(row)
    {
    case 2:
      {
    column = line2Column;
    colIndex = line2BufIndex;
    lineBuffer= line2Buffer;
    break;
      }
    case 3:
      {
    column = line3Column;
    colIndex = line3BufIndex;
    lineBuffer= line3Buffer;
    break;
      }

      // to elimnate a compile time warning
    default:
      {
    column = line2Column;
    colIndex = line2BufIndex;
    lineBuffer= line2Buffer;
    break;
      }
    }


  // See if character is backspace:
  if (newChar == 8)
    {
      if (column != 0)   column--;
      if (colIndex != 0) colIndex--;
      newChar = ' ';
      backspace = 1;
    }
  else
    {
      backspace = 0;
    }


  sprintf(tempString, "%c", newChar);
  if (column < 20)
    {
      lcd.locate(column++,row);
      lcd.printf(tempString);
      lineBuffer[colIndex++] = newChar;
      if (backspace)
    {
      column--;
      colIndex--;
    }
    }
  else
    {
      // Shift row 2 text left one position
      // First, do it in the buffer
      destIndex = 0;
      for (srcIndex = 1; srcIndex <= 19; srcIndex++)
    {
      lineBuffer[destIndex++] = lineBuffer[srcIndex];
    }
      lineBuffer[19] = 0;  // end of string
      // Now do it on the LCD
      lcd.locate(0,row);
      lcd.printf(lineBuffer);
      lcd.locate(19,row);
      lcd.printf(tempString);
      lineBuffer[19] = newChar;
    }

  switch(row)
    {
    case 2:
      {
    line2Column = column;
    line2BufIndex = colIndex;
    break;
      }
    case 3:
      {
    line3Column = column;
    line3BufIndex = colIndex;
    break;
      }
    }
}


/** -------------------------------------------------------------------------

@verbatim
Change weight as a function of the arrow key control.

@endverbatim
**/

void deltaWeight(int delta)
{
  weight += delta;

  calcTimeDurationsKbd(kbdWPM, weight);
  calcTimeDurationsPdl(pdlWPM, weight);

  displaySpeedsLCD();
}



/** -------------------------------------------------------------------------
@verbatim
Display the current ten minute timer state, with visual warnings 
from 9 through 12 minutes.

@endverbatim

@param [in] minutes  Minutes count for ID timer
@param [in] seconds  Seconds count for ID timer
@param [in] timeTicIntrCount  0 = 1st half second interval 1 = 2nd half

**/

void displayTimerLCD(int minutes, int seconds, int timeTicIntrCount)
{
  lcd.locate(TIMEcursor);

  if ((minutes < 9) || (minutes >12))
    {
      lcd.printf("%02d:%02d", minutes, seconds);
    }
    else
      {
    // Toggle the ** with seconds every half second
    if (timeTicIntrCount == 0)
      {
        lcd.printf("%02d:**", minutes);
      }
    else
      {
        lcd.printf("%02d:%02d", minutes, seconds);
      }        
      }
}

