/** -------------------------------------------------------------------------
Copyright (c) 2012 Chuck Broadwell W5UXH

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

**/

#include "mbed.h"
#include "USBHost.h"
#include "Utils.h"
#include "w5uxhMbedKeyer.h"
#include "queues.h"

extern Fifo *cwBuf;
extern Fifo *consoleBuf;
extern Serial pc;

extern  unsigned char scancodeToAsciiLookupTable[4][128];

// The number of keys we will keep track of for press / release
#define N_KEYS 6

unsigned int activeKeys[N_KEYS];
extern void (*ctrlFuncs[128])(char asciiChar);

// We check byte 0 of the USB data packet and determine which section of 
// the scancode table to use.
#define NORM  0x00
#define CTRL  0x01
#define SHIFT 0x02
#define ALT   0x03
#define GUI   0x00

/** -------------------------------------------------------------------------
@verbatim
We have a new key press so we lookup the key code to ASCII (or command code) and
write the result in the appropriate buffer.
Initially, we just put everything in the kbd cw buffer.

@endverbatim
**/
void handleNewKeyPress(u8 byte0, u8 newPacketCode)
{
  char asciiChar;  
  int  shiftCode;

  switch(byte0)
    {
    case 0x00:            shiftCode = NORM;  break;
    case 0x01: case 0x10: shiftCode = CTRL;  break;
    case 0x02: case 0x20: shiftCode = SHIFT; break;
    case 0x04: case 0x40: shiftCode = ALT;   break;
    case 0x08: case 0x80: shiftCode = GUI;   break;
    default: shiftCode = NORM; break;
    }


  asciiChar = scancodeToAsciiLookupTable[shiftCode][newPacketCode];

  // Only backspace if there is still data in the buffer.
  if ((asciiChar == 0x08) && (cwBuf->avail()))
    {
      cwBuf->backspace();
      pc.printf("\b \b");
      displayLcdTextRow(2,asciiChar);
    }
  else
    {
      // eventually need to put control codes in the command queue, but for a test, check for the
      // arrow keys and handle here

      // 0x80 is not used for a key assignment, it is the flag for end of msg in the buffer
      // 0x81-0x84 are not currently used for commands, they are the AS, VE, AR, and SK prosigns.
      // I still need to handle prosigns (including BT and AA) on the LCD and console displays.
      if (asciiChar > 0x80)
	{
	  (*ctrlFuncs[asciiChar-0x80])(asciiChar);
	}
      else
	{
	  if ((asciiChar == 0x0D) || (asciiChar == 0x09))
	    {
	      dsplyProsign(asciiChar);
	    }
	  else
	    {
	      if (asciiChar != 0x08)
		{
		  if (cwBuf->write(asciiChar)) consoleBuf->write(asciiChar);
		}
	      else
		{
		  if (cwBuf->avail()) consoleBuf->write(asciiChar);
		}
	    }
	}
    }
}


/** -------------------------------------------------------------------------
@verbatim
Clear the active list, perhaps this only happens when all zero packet is received.

@endverbatim
**/
void clearActiveList(void)
{
  int activeKeysIndex;

  for(activeKeysIndex = 0; activeKeysIndex < N_KEYS; activeKeysIndex++)
    {
      activeKeys[activeKeysIndex] = 0;
    }
}

/** -------------------------------------------------------------------------
@verbatim
Store the new key press code in the active list, if there is room for it.

@endverbatim
**/
bool storeCodeInActiveList(u8 newKeyPressCode)
{
  int activeKeysIndex;

  for(activeKeysIndex = 0; activeKeysIndex < N_KEYS; activeKeysIndex++)
    {
      // If we exit the loop before reaching the limit, we store the new code
      // in the list to flag the key as being pressed, otherwise we ignore the key.
      // If successful in storing the code, we return TRUE, otherwise FALSE.
      if(activeKeys[activeKeysIndex] == 0) break;
    }
  if (activeKeysIndex != N_KEYS)
    {
      activeKeys[activeKeysIndex] = newKeyPressCode;
      return TRUE;
    }
  else
    {
      return FALSE;
    }
}

/** -------------------------------------------------------------------------
@verbatim
We check the codes in the new packet from the USB keyboard to the list of keys
currently defined as being pressed.  If any of the new codes are not currently
listed in the activeKeys array, then it is a new key press.  We need to store
the code in the array and convert the code to ASCII (or a control code) and
write the result in the appropriate buffer (kbd buffer or command buffer).

The data packet length should always be 8, where element 0 is the modifier byte,
element 1 is the OEM (unused) byte, and elements 2-7 are the potential key codes.
At a minimum, when we reach here, element 2 should be non-zero.  If multiple keys
were pressed when the packet was generated, then successive elements will also be
non-zero.

@endverbatim
**/
void checkForKeyPress(u8* data)
{
  int newPacketIndex;
  int activeKeysIndex;
  u8  newPacketCode;
  bool result;

  // See if any code in the new data packet is currently active (pressed)
  for(newPacketIndex = 2; newPacketIndex < 8; newPacketIndex++)
    {
      newPacketCode = data[newPacketIndex];
      for(activeKeysIndex=0; activeKeysIndex<N_KEYS; activeKeysIndex++)
    {
      // If the code in new packet is already shown as active, we ignore the code.
      if (newPacketCode == activeKeys[activeKeysIndex]) break;
    }
      
      // If we exit the loop before reaching the limit, we ignore the code. Otherwise
      // this is a new key press so we need to use the code.
      if (activeKeysIndex == N_KEYS)
    {
      result = storeCodeInActiveList(newPacketCode);
      // If there is no room in the active list (FALSE), we will ignore this code
      // otherwise:
      if (result)
        {
          // We stored the code in the active list successfully.
          handleNewKeyPress(data[0], newPacketCode);
        }
    }
    }
}


/** -------------------------------------------------------------------------
@verbatim
We check every code in the active list to see if the same code is still in the
data packet from the USB keyboard.  If the code is not found in the data packet,
then we remove the code from the active list (this is key release).  If the code
is still in the packet, then the key is still pressed.

@endverbatim
**/
void checkForKeyRelease(u8* data)
{
  int activeKeysIndex;
  int newPacketIndex;
  u8  activeKeyCode;

  for(activeKeysIndex=0; activeKeysIndex<N_KEYS; activeKeysIndex++)
    {
      activeKeyCode = activeKeys[activeKeysIndex];
      if (activeKeyCode != 0)
    {
      for(newPacketIndex=2; newPacketIndex<8; newPacketIndex++)
        {
          if(activeKeyCode == data[newPacketIndex]) break;
        }
      // If we exit the loop above early, then the active key code
      // is still in the newest packet so it is still pressed.
      // If the code is not found in the packet:
      if (newPacketIndex == 8)
        {
          // then the key has been released and we can remove it from the active list.
          activeKeys[activeKeysIndex] = 0;
        }
    }
    }
}


/** -------------------------------------------------------------------------
@verbatim
AutoEventCallback from USBHostShell calls this custom handler for the USB 
keyboard.

@endverbatim
**/
void keyerAutoEventKbdCallback(int len, u8* data)
{
  int dataSum = 0;
  int tempLen = len;
  const u8* tempData = data;


  // Check for non-zero bytes in elements beyond element 0 (the modifier byte)
  tempData++;
  tempLen--;
  while(tempLen--   >   0)
    {
      dataSum += *tempData++;
    }

  //  if (dataSum != 0) printfBytes("data",data,len);
  

  // I mostly ignore the len parameter since it should always be 8.
  if (dataSum!=0)
    {
      checkForKeyPress(data);
      checkForKeyRelease(data);
    }
  else
    {
      // Clear the active key list, no keys pressed
      clearActiveList();
    }
}
