#include "mbed.h"
#include <new>
#include "queues.h"
#include "w5uxhMbedKeyer.h"

/** 
I copied fifoBuf.cpp to queues.cpp to use it as a template for writing classes "Fifo".

I will initially use it for a new queue:  FifoS  consoleBuf;  
where I currently do not use a queue.  I just use a flag for the idle loop, so it only handles
a single character.  

Eventually:

rxDataRcvd and keyerKbdCallback will use these buffers:

    Fifo   cwBuf        (previously was kbdBuf)
    Fifo   lcdBuf       (previously was inBuf)
    Fifo   consoleBuf   (new buf)
    Fifo   cmdBuf       (same name as before)

**/

extern Serial pc;
extern Fifo *cwBuf;


//////////////////////////////////////////////////////////////////////////
// Start of public functions for class Fifo

/** -------------------------------------------------------------------------
@verbatim
Public member functions for class Fifo

@endverbatim
**/
Fifo::Fifo(int q_size, char *fifoName)
{
  q = new (nothrow) char[q_size];
  if (q != NULL)
    {
      size = q_size;
      in = out = count =0;

      //pc.printf("Allocated %d byte buffer for %s\r", q_size, fifoName);
    }
  else
    {
      pc.printf("Failed to allocate %d byte buffer for %s\r", q_size, fifoName);
    }
}

bool Fifo:: write(char data)
{
  __disable_irq();
  
  if (count == (size-1))
    {
      // Buffer is full, ignore
      __enable_irq();
      return FALSE;
    }
  else
  {
    q[in] = data;
    count++;
    if (++in >= size)
     {
      in = 0;
     }
    statusFlag = TRUE;
  }
  __enable_irq();
  return TRUE;
}

char Fifo::read(void)
{
  char data;

  __disable_irq();
  data = q[out];
  count--;
  if (++out >= size)
    {
      out = 0;
    }
  statusFlag = TRUE;
  __enable_irq();    
  return data;
}

bool Fifo::avail(void)
{
  bool result;

  __disable_irq();
  if (in != out)
    {
      result= TRUE;
    }
  else
    {
      result= FALSE;
    }
  __enable_irq();
  
  return result;
}

int Fifo::get_size(void)
{
  return size;
}

int Fifo::get_count(void)
{
  return count;
}

void Fifo::reset(void)
{
  __disable_irq();
  in    = 0;
  out   = 0;
  count = 0;
  __enable_irq();
  dsplyBufStatusLCD();
  dsplyBufStatusSerial(NO_FORCE);
}

void Fifo::backspace(void)
{
  __disable_irq();  
  if(--in < 0) in = size;
  if(--count < 0) count=0;
  __enable_irq();
}

bool Fifo::getStatusFlag(void)
{
  return statusFlag;
}

void Fifo::clearStatusFlag(void)
{
  statusFlag = FALSE;
}

// End of class Fifo public functions.
//////////////////////////////////////////////////////////////////////////



/** -------------------------------------------------------------------------
@verbatim
When displaying characters at the output of the cwBuf, I only display them on
the bottom line (line 3) of the LCD.  Line 2 of the LCD and the console window
already have the characters from when they were input.  If there ever is a 
second console window for this purpose, then that will be handled here also.

@endverbatim
**/
void dsplyBufOutput(char outChar)
{
  char c0 = '?';
  char c1 = '?';

  if ((outChar >= 0x20) && (outChar < 0x80))
    {
      displayLcdTextRow(3,outChar);
    }
  else
    {
      switch (outChar)
    {
    case AS: c0='A'; c1='S';  break;
    case VE: c0='V'; c1='E';  break;
    case AR: c0='A'; c1='R';  break;
    case SK: c0='S'; c1='K';  break;
    case AA: c0='A'; c1='A';  break;
    case BT: c0='B'; c1='T';  break;
    }      
      displayLcdTextRow(3,' ');
      displayLcdTextRow(3,c0);
      displayLcdTextRow(3,c1);
      // The extra space in the cwBuf means we do not need to add one here.
      // Part of the general confusion, but I believe this is correct.
  //    displayLcdTextRow(3,' ');
    }
}

extern unsigned char kbdWPM;
extern unsigned char weight;
extern unsigned char lcdDisplayState;

int cannedMsgState= IDLE;

/** -------------------------------------------------------------------------
@verbatim
Check to see if data is available in the cw buffer;  If so, read the next
character and start sending the new character.  Also display char on
bottom row of LCD, and check for ID detection / timer reset.

@endverbatim
**/
void checkCwBuffer(void)
{
  char inChar;
  unsigned char morseByte;


  if (cwBuf->avail())
    {
      inChar = cwBuf->read();

      // Check for control codes, handle and replace with SPACE
      switch(inChar)
	{
	  // restore original kbd speed if end of announce msg
	case EOM_RESTORE_KBD_SPD: 
	  {
	    calcTimeDurationsKbd(kbdWPM, weight); 
	    inChar = ' '; 
	    break;
	  }

	case CANNED_MSG_START:
	  {
	    cannedMsgState = ACTIVE;
	    break;
	  }

	case CANNED_MSG_END:
	  {
	    cannedMsgState = IDLE;
	    break;
	  }
	}

      morseByte = asciiToMorseLookup(inChar);
      startCwCharacter(morseByte);  
      checkId(inChar);
      
      // kludge to handle the Enter key from the console
      // Somewhere before the buffer input, this key from console
      // is translated to 0x3D (=) for BT, and somewhere else BT is
      // defined as 0x0D (CR);  all very confusing
      if (inChar=='=') inChar=0x0D;

      if (cannedMsgState == ACTIVE)
	{
	  if ((inChar >= 0x20) && (inChar < 0x80))
	  displayLcdTextRow(2,inChar);
	}
      else
	{
	  if ((inChar != 0x08)&& (lcdDisplayState == BOTH_ROWS)) 
	    {
	      dsplyBufOutput(inChar);
	    }
	}
    }

}

